#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid.forms.formalchemy.renderers`` -- Field Renderers
"""

import datetime

import formalchemy

from webhelpers.html import literal

from pyramid.renderers import render

from rattail.barcodes import GPC
from rattail.pyramid.forms.core import pretty_datetime
from rattail.pyramid.forms.formalchemy.fieldset import FieldSet
from rattail.time import local_time


__all__ = ['AutocompleteFieldRenderer', 'EnumFieldRenderer',
           'StrippingFieldRenderer', 'YesNoFieldRenderer',
           'DateTimeFieldRenderer']


def AutocompleteFieldRenderer(service_url, field_value=None, field_display=None, width='300px'):
    """
    Autocomplete renderer.
    """

    class AutocompleteFieldRenderer(formalchemy.fields.FieldRenderer):

        @property
        def focus_name(self):
            return self.name + '-textbox'

        @property
        def needs_focus(self):
            return not bool(self.value or field_value)

        def render(self, **kwargs):
            kwargs.setdefault('field_name', self.name)
            kwargs.setdefault('field_value', self.value or field_value)
            kwargs.setdefault('field_display', self.raw_value or field_display)
            kwargs.setdefault('service_url', service_url)
            kwargs.setdefault('width', width)
            return render('/forms/field_autocomplete.mako', kwargs)

    return AutocompleteFieldRenderer


def EnumFieldRenderer(enum):
    """
    Adds support for enumeration fields.
    """

    class Renderer(formalchemy.fields.SelectFieldRenderer):
        
        def render_readonly(self, **kwargs):
            value = self.raw_value
            if value is None:
                return ''
            if value in enum:
                return enum[value]
            return str(value)

        def render(self, **kwargs):
            opts = [(enum[x], x) for x in sorted(enum)]
            return formalchemy.fields.SelectFieldRenderer.render(self, opts, **kwargs)

    return Renderer


class StrippingFieldRenderer(formalchemy.TextFieldRenderer):
    """
    Standard text field renderer, which strips whitespace from either end of
    the input value on deserialization.
    """

    def deserialize(self):
        value = super(StrippingFieldRenderer, self).deserialize()
        return value.strip()


class YesNoFieldRenderer(formalchemy.fields.CheckBoxFieldRenderer):

    def render_readonly(self, **kwargs):
        value = self.raw_value
        if value is None:
            return ''
        return 'Yes' if value else 'No'


class DateTimeFieldRenderer(formalchemy.fields.DateTimeFieldRenderer):
    """
    Leverages Rattail time system to coerce timestamp to local time zone before
    displaying it.
    """

    def render_readonly(self, **kwargs):
        value = self.raw_value
        if isinstance(value, datetime.datetime):
            value = local_time(value)
            return value.strftime(self.format)
        return ''

FieldSet.default_renderers[formalchemy.types.DateTime] = DateTimeFieldRenderer


def PrettyDateTimeFieldRenderer(from_='local', to='local'):

    class PrettyDateTimeFieldRenderer(formalchemy.fields.DateTimeFieldRenderer):

        def render_readonly(self, **kwargs):
            return pretty_datetime(self.raw_value, from_=from_, to=to)

    return PrettyDateTimeFieldRenderer


class GPCFieldRenderer(formalchemy.TextFieldRenderer):
    """
    Renderer for :class:`rattail.GPC` fields.
    """

    @property
    def length(self):
        # Hm, should maybe consider hard-coding this...?
        return len(str(GPC(0)))


class PriceFieldRenderer(formalchemy.TextFieldRenderer):
    """
    Renderer for fields which reference a :class:`ProductPrice` instance.
    """

    def render_readonly(self, **kwargs):
        price = self.field.raw_value
        if price:
            if price.price is not None and price.pack_price is not None:
                if price.multiple > 1:
                    return literal('$ %0.2f / %u&nbsp; ($ %0.2f / %u)' % (
                            price.price, price.multiple,
                            price.pack_price, price.pack_multiple))
                return literal('$ %0.2f&nbsp; ($ %0.2f / %u)' % (
                        price.price, price.pack_price, price.pack_multiple))
            if price.price is not None:
                if price.multiple > 1:
                    return '$ %0.2f / %u' % (price.price, price.multiple)
                return '$ %0.2f' % price.price
            if price.pack_price is not None:
                return '$ %0.2f / %u' % (price.pack_price, price.pack_multiple)
        return ''


class PriceWithExpirationFieldRenderer(PriceFieldRenderer):
    """
    Price field renderer which also displays the expiration date, if present.
    """

    def render_readonly(self, **kwargs):
        res = super(PriceWithExpirationFieldRenderer, self).render_readonly(**kwargs)
        if res:
            price = self.field.raw_value
            if price.ends:
                res += '&nbsp; (%s)' % pretty_datetime(price.ends, from_='utc')
        return res
